CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC    
C                                                                             C
C      Program FITEST1                                                        C
C                                                                             C
C      This is a program for test of numerical evaluation                     C
C      of functional integrals using FUNINT subroutine                        C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC    
     
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(4),AX(4),BX(4)
      COMMON/PAR/B,DX(2),L,M,H
    
      B=1.D0
      N=3
      K=N+1
      XM=4.D0
      EPS=1.D-3

C      definition of parameters for MIKOR
     
      M=1                ! number of integrals
      DX(1)=-1.D0        ! initial number of integration points (auto) 
      DX(2)=-1.D0        ! final number of integration points (auto) 
      L=-1               ! periodization order (auto)
      H=-1.D0            ! boundary layer thickness (auto)
     
     
      WRITE(*,*) 'Test of the functional integration using FUNINT' 
      WRITE(*,*) 
      WRITE(*,*) '  F[x] = EXP(-1/2 integral[0,1] X(t)**2 dt)   ' 
      WRITE(*,*) 
      WRITE(*,*) '   R_exact = 1/SQRT(SINH(1)) =  0.922452237...   '  
      WRITE(*,*) 
     
      CALL FUNINT(K,X,AX,BX,XM,EPS,R)
     
      WRITE(*,*) '  n = ',N, '  R_n = ', R
      WRITE(*,*) 
     
        
      END
     
     
      SUBROUTINE FUNINT(K,X,AX,BX,XM,EPS,R)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC    
C                                                                             C
C      This program performs numerical evaluation of functional integrals     C
C      with Wiener measure from the real functional of exponential form       C
C                                                                             C
C                                                                             C
C      The program uses subroutine MIKOR for calculation of the               C
C      multidimensional Riemann integral by the Korobov method                C
C                                                                             C
C                                                                             C
C      The program uses external module DSIMPS for calculation of the         C
C      integral over the auxiliary variable tau by the Simpson's method       C
C                                                                             C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
     
      IMPLICIT REAL*8(A-H,O-Z)
      EXTERNAL FX
      DIMENSION X(K),AX(K),BX(K),F(1),RES(1),DX(2),REPS(1)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      
      
      KK=K
      PI=4.D0*DATAN(1.D0)
      EPS1=DABS(EPS)
      N=K-1

C       Definition of parameters for calculation of
C       (N+1)-dimensional integral using MIKOR 

      KR=K               ! total multiplicity
      M=MM               ! number of integrals
      DO 10 J=1,2        !
      DX(J)=DDX(J)       ! initial number of integration points 
10    CONTINUE           !
      L=LL               ! periodization order
      H=HH               ! boundary layer thickness
      DO 20 I=1,N        !
      AX(I)=-DABS(XM)    ! lower integration limits for U
      BX(I)=DABS(XM)     ! upper integration limits for U
20    CONTINUE           !
      AX(KK)=-1.D0       ! lower integration limits for V
      BX(KK)=1.D0        ! upper integration limits for V
                         
      REPS(1)=-EPS1      ! relative accuracy
      
      CALL MIKOR(KR,M,X,F,RES,DX,L,H,REPS,FLAG,FX,AX,BX)
      
      R=0.5D0*RES(1)/DSQRT(2.D0*PI)**N
      
      RETURN
      END

C      Definition of integrand for MIKOR
      SUBROUTINE FX(K,M,X,F,R,EPS,FLAG)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(K),F(M),R(M)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      
      N=K-1
      
      ARG=0.D0
      DO 30 I=1,N
      ARG=ARG+X(I)**2
30    CONTINUE
      ARG=-0.5D0*ARG

      E=DEXP(ARG)

C      Calculation of the function at the necessary points 
      FF=FUNC(K,X)
      
      F(1)=E*FF
      
      RETURN
      END

C      Definition of the finctional F
      FUNCTION FUNC(K,X)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(K)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/U(20),V(1)
      EXTERNAL FUN1,FUN2
      
      A1=0.D0
      B1=1.D0
      H1=1.D-2
      REPS1=1.D-4
      AEPS1=1.D-10
      
      N=K-1
      DO 40 I=1,N
      U(I)=X(I)
40    CONTINUE
      V(1)=X(K)

      CALL DSIMPS(A1,B1,H1,REPS1,AEPS1,FUN1,T1,AI1,AIH1,AIABS1)
      
      A2=0.D0
      B2=1.D0
      H2=1.D-2
      REPS2=1.D-4
      AEPS2=1.D-10
      
      CALL DSIMPS(A2,B2,H2,REPS2,AEPS2,FUN2,T2,AI2,AIH2,AIABS2)
      
      E=DEXP(-BB*AI2)
      
      FUNC=AI1*E
      
      RETURN
      END
      
C      Calculation of the integral part of the functional
      FUNCTION FUN1(T)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/CONST/K,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/UU(20),VV(1)
      
      N=K-1
      V=VV(1)
      
      SGV=DSIGN(1.D0,V)
      RVT=RHO(V,T)
      
      SN=0.D0
      UN=0.D0
      
      DO 50 J=1,N
      DJ=J
      SJPT=DSIN(DJ*PI*T)
      CJPV=DCOS(DJ*PI*V)
      SN=SN+SJPT*CJPV/DJ
      UN=UN+UU(J)*SJPT/DJ
50    CONTINUE

      SN=SN*2.D0*SGV/PI
      UN=UN*DSQRT(2.D0)/PI
      
      RSV=RVT-SN+UN

      FUN1=F1(RSV)

      RETURN
      END

C      Calculation of exponential part of the functional
      FUNCTION FUN2(T)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/CONST/K,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/UU(20),VV(1)
      
      N=K-1
      V=VV(1)
      
      SGV=DSIGN(1.D0,V)
      RVT=RHO(V,T)
      
      SN=0.D0
      UN=0.D0
      
      DO 60 J=1,N
      DJ=J
      SJPT=DSIN(DJ*PI*T)
      CJPV=DCOS(DJ*PI*V)
      SN=SN+SJPT*CJPV/DJ
      UN=UN+UU(J)*SJPT/DJ
60    CONTINUE

      SN=SN*2.D0*SGV/PI
      UN=UN*DSQRT(2.D0)/PI
      
      RSV=RVT-SN+UN
      
      FUN2=F2(RSV)

      RETURN
      END

C      Auxilary function for functional integration
      FUNCTION RHO(V,T)
      IMPLICIT REAL*8(A-H,O-Z)
      
      AV=DABS(V)
      SGV=DSIGN(1.D0,V)
        
      IF (T.LE.AV) GOTO 1 
      R=(1.d0-T)*SGV
      GOTO 2
1     CONTINUE
      R=-T*SGV
2     RHO=R
      
      RETURN
      END
      

C      User's function
      FUNCTION F1(X)
      IMPLICIT REAL*8(A-H,O-Z)

      F1=1.d0

      RETURN
      END

C      User's function
      FUNCTION F2(X)
      IMPLICIT REAL*8(A-H,O-Z)

      F2=0.5d0*X**2

      RETURN
      END

C      ===================================================================


      SUBROUTINE MIKOR(K,N,X,F,R,T,L,H,EPS,FLAG,EXTERN,AX,BX)
      IMPLICIT REAL*8(A-H,O-Z)
      EXTERNAL EXTERN
      COMMON/MIKINK/KDIM,KDAB,P/TIMEMI/TIMEM
      DIMENSION X(K),F(N),EPS(N),AX(K),BX(K),T(2),R(N)

      KDIM=1
      KDAB=1
   2  FORMAT(/28H THE REQUIRED PRECISION EPS=,E10.3,15H IS NOT ARRIVED,
     135H IT IS NECESSARY T(1) GREATER THEN=,F8.0,17H TO ASSIGN OR THE,
     26H VALUE/15H OF L TO CHANGE,
     3/ 46H THE ACTUAL VALUES OF ERRORS LOWER ARE PRINTED)
    3 FORMAT (10D12.4)
   12 FORMAT(' THE REQUIRED PRECISION EPS(N) IS NOT ARRIVED:'/(10E12.5))
   13 FORMAT(' IT IS NECESSARY T(1) GREATER THEN=',F8.0,' TO ASSIGN,OR
     1 THE VALUE OF L TO CHANGE'/
     2' THE ACTUAL VALUES OF ERRORS LOWER ARE PRINTED')
      CALL DATKOR(K,0.D0)
      CALL INKOR(K,N,X,F,R,T,L,H,EPS,FLAG,EXTERN,AX,BX)
      IF(FLAG)10,1,10
    1 CONTINUE
      IF (KDIM.NE.1) GO TO 20
      PRINT 2, EPS(1),P
      GO TO 21
   20 CONTINUE
      PRINT 12, EPS
      PRINT 13,P
   21 CONTINUE
      N1=N+1
      N2=N+N
      PRINT 3, (R(I),I=N1,N2)
  10  CONTINUE
      RETURN
      END

      SUBROUTINE DATKOR(K,DPRINT)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/INDAT/ PT(30),AT(30),LT
      DIMENSION KP(27),KP1(26),KA(260),KA1(103),KA104(78),KA182(69)
      EQUIVALENCE (KA(1),KA1),(KA(104),KA104),(KA(182),KA182)
      DATA KAMAX/400000/
      DATA LKP/27/, LKP1/26/, KMAX/20/
      DATA KP/23,53,101,151,307,523,829,1259,2129,3001,4001,5003,6007,
     A8191,10007,13001,20011,30011,40009,50021,75011,100003,200003,
     B500009,1000003,2000003,5000011/
      DATA KP1/3,5,8,13,21,34,55,89,144,233,377,610,987,1597,2584,4181,
     A6765,10946,17711,28657,46368,75025,121393,196418,317811,514229/
      DATA KA1/
     3   5,17,53,20,75,78,116,433,359,276,722,1476,592,739,544,2135,
     3   2759,2023,2979,3483,7623,6697,19477,55279,90347,191421,613511,
     4   5,31,15,22,42,178,45,483,766,174,113,792,1351,2488,1206,778,
     4 2891,2243,2775,3353,7601,10101,19293,55251,122713,191447,492615,
     5   21,13,127,64,146,302,618,408,1534,840,509,1386,198,830,2803,
     5   2311,2945,3581,7639,10019,19777,61357,122713,191447,492615,
     6   109,195,118,109,200,41,233,1751,2037,312,1632,2240,1940,3169,
     6   2347,2875,3431,7647,10049,19277,33363,66681,191401,613509/
      DATA KA104/
     7   27,90,227,596,170,115,229,721,2107,2304,2116,1773,2253,2965,
     7   3401,7555,6703,24561,33339,66671,191443,613509,
     8   32,392,86,993,1850,2362,1253,3787,436,1415,3295,3267,3093,
     8   3455,7809,10029,19827,61365,90343,191437,492261,
     9   636,108,1579,526,1597,2139,470,1437,2533,2077,3119,3469,7813,
     9   10099,19461,61365,90343,191415,613507,
     *   431,1262,1751,564,1415,1715,2065,2901,3413,7677,6699,19153,
     *   33341,107369,191435,613515/
      DATA KA182/349,
     1   586,1409,2205,3237,4325,5419,8063,10151,24573,55269,107393,
     1   191389,613515,
     2 2191,3319,4319,3407,8099,10151,24573,61365,107377,191389,613507,
     3   5407,8061,10761,24573,61371,107375,191433,478471,
     4   5407,8099,10143,19363,61371,106245,191433,478471,
     5   5417,8061,10171,20015,61371,106265,191405,478471,
     6   61373,106257,191405,492613,
     7   61353,106245,191421,492613,
     8   61353,106257,191421,478471,
     9   61353,106245,191421,478471,
     *   61353,106233,191421,478471/
      DO 30 I=1,30
      PT(I)=0.
  30  AT(I)=0.
      KR=K
      IF(KR-KMAX)2,2,1
   1  PRINT 101
 101  FORMAT(27H SUBROUTINE MIKOR OR MIKOR2
     1/ 36H THE VALUE OF K MUST BE LESS THEN 21)
      STOP
   2  IF(KR-3)3,5,5
   3  LT=LKP1-1
      DO 4 I=1,LT
      J=I+1
      PT(I)=KP1(J)
      AT(I)=KP1(I)
   4  CONTINUE
      GO TO 20
   5  CONTINUE
      L=2
      I=0
      GO TO 9
   6  LT=0
   7  I=I+1
      LT=LT+1
      IF(KA(I)-KAMAX)7,8,8
   8  L=L+1
   9  IF(L-KR)6,10,10
  10  I=I-LT
      J=LKP-LT
      DO 11 IJ=1,LT
      I=I+1
      J=J+1
      PT(IJ)=KP(J)
  11  AT(IJ)=KA(I)
  20  CONTINUE
      IF(DPRINT)21,23,21
  21  CONTINUE
      LTP=LT
      DO 50 IPR=1,100,10
      LP=MIN0(LTP,10)
      KPR=IPR+LP-1
      LTP=LTP-10
      IF(LTP)51,51,50
  50  CONTINUE
  51  CONTINUE
  23  CONTINUE
      RETURN
 102  FORMAT(/40X,30H THE PARAMETERS OF THE KOROBOV,2H,S ,
     112H NETS FOR K=,I2)
      END

      SUBROUTINE INKOR(K,N,X,F,R,T,L,H,EPS,FLAG,EXTERN,AX,BX)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/MIKINK/KDIM,KDAB,P/TIMEMI/TIMEM/INDAT/PT(30),AT(30),LT
      DIMENSION X(K),F(N),R(N)  ,S(20),S1(20),CSIG(11),CTAU(11)
      DIMENSION AX(K),BX(K),T(2),EPS(N),AD(2)
      INTEGER TIN, TEN
      EQUIVALENCE(D,AD)
      DATA LMAX/10/, RFLAG/1.E6/, CP3/10./, CMAX/20./
      ITM=2
      KR=K
      LP=L
      N2=N+N
      N3=N2+N
      N21=N2+1
      P3=3.**KR
      PMAX=T(2)
      IF(PMAX)1,2,300
   1  PMAX=CMAX*P3
      GO TO 300
   2  PMAX=PT(LT)
 300  CONTINUE
      TIME=TIMEM
      IF(TIME)301,301,302
 301  TIME=1.E12
 302  CNTP=0.
      Q=T(1)
      IF(Q)7,9,9
   7  CONTINUE
      IF(KR-10)8,8,9
   8  CONTINUE
      Q=P3
   9  CONTINUE
      P3=CP3*P3
      IF(Q-PT(LT))4,4,3
   3  Q=PT(LT)
   4  IF(LP-LMAX)6,6,5
   5  LP=LMAX
   6  DO 999 KT=1,LT
      FLAG=0.
      P=PT(KT)
      A=AT(KT)
      IF(Q-P)10,10,999
  10  Z=2.*P
 401  CONTINUE
 402  ITM=1
      CNTP=CNTP+P
      LFLAG=2
      IF(LP)14,19,19
  14  LP=0
      IF(Q-P3)16,15,15
  15  LP=1
  16  IF(KR-6)17,17,19
  17  LP=LP+1
  19  CONTINUE
      IF(LP)20,20,11
  11  LFLAG=1
      CALL PERIOD(KR,P,H,LP,ALPHA)
      ALPHA1=1./ALPHA
      ALPHA2=0.5*ALPHA
      BETA=1.-ALPHA
      BETA1=1./BETA
      L1=LP+1
      AK=0.
      AL=L1
      A2L=LP+L1
      B2L=A2L*AL/(AL-1.)
      CL=1.
      C2L=1.
      DO 12 I=1,L1
      CTAU(I)=CL/A2L
      CSIG(I)=CTAU(I)/(A2L+1.)
      AK=AK+1.
      AL=AL-1.
      A2L=A2L-1.
      CL=CL*AL/AK
      C2L=C2L*A2L/AK
  12  CONTINUE
      B2L=B2L*C2L*BETA1
      DO 13 I=1,L1
      CTAU(I)=B2L*CTAU(I)
      CSIG(I)=B2L*ALPHA*CSIG(I)
  13  CONTINUE
  20  A1=1.
      DO 23 JK=1,KR
      IF (DMOD(A1,2.D0)) 21,21,22
  21  A1=P-A1
  22  S1(JK)=A1
      S(JK)=S1(JK)*2.
      A1=DMOD(A1*A,P)
  23  CONTINUE
      DO 24 IJ=1,N2
  24  R(IJ)=0.
      JN=0
      M=P
      DO  100 I=1,2
      GO TO (29,28),KDAB
  28  CONTINUE
      DO 30 IJ=N21,N3
  30  R(IJ)=0.
  29  CONTINUE
      DO 90 J=1,M
      FACT=1.
      F1=1.
      DO 70 JK=1,KR
      Y1=S1(JK)/Z
      S1(JK)=S1(JK)+S(JK)-Z
      IF(S1(JK))31,32,32
  31  S1(JK)=S1(JK)+Z
  32  GO TO (33,61),LFLAG
  33  W=Y1
      Y=W*ALPHA1
      IF(W-ALPHA)50,40,40
  40  IF(W-BETA)41,41,42
  41  Y1=(W-ALPHA2)*BETA1
      TAU=BETA1
      GO TO 60
  42  Y=ALPHA1-Y
  50  Y1=-Y
      Y2=Y
      SIG=CSIG(1)
      TAU=CTAU(1)
      DO 51 IL=2,L1
      Y2=Y2*Y
      SIG=SIG*Y1+CSIG(IL)
      TAU=TAU*Y1+CTAU(IL)
  51  CONTINUE
      Y1=SIG*Y2*Y
      IF(Y1)53,54,54
  53  CONTINUE
      Y1=0.
  54  CONTINUE
      TAU=TAU*Y2
      IF(W-ALPHA)60,52,52
   52 Y1=1.-Y1
  60  FACT=FACT*TAU
  61  CONTINUE
      Y=BX(JK)-AX(JK)
      X(JK)=Y*Y1+AX(JK)
      GO TO (63,64),KDAB
  63  CONTINUE
      FACT=FACT*Y
      GO TO 70
   64 F1=F1*Y
  70  CONTINUE
      CALL EXTERN(K,N,X,F,R,EPS,FLAG)
CALL USER'S SUBROUTINE
      DO 89 IJ=1,N
      IN=IJ+JN
      J2=IJ+N2
      GO TO (83,84),KDAB
  83  R(IN)=R(IN)+FACT*F(IJ)
      GO TO 89
  84  F1=F1*FACT*F(IJ)
      AD(1)=R(IN)
      AD(2)=R(J2)
      D=D+F1
      R(IN)=AD(1)
      R(J2)=AD(2)
  89  CONTINUE
  90  CONTINUE
      DO 91 JK=1,KR
      S1(JK)=S(JK)
  91  CONTINUE
      DO 99 IJ=1,N
      IN=IJ+JN
      J2=IJ+N2
      GO TO (93,94),KDAB
  93  R(IN)=R(IN)/P
      GO TO 99
  94  AD(1)=R(IN)
      AD(2)=R(J2)
      R(IN)=D/P
  99  CONTINUE
      JN=JN+N
 100  CONTINUE
      KFLAG=1
      EPS1=EPS(1)
      DO 200 I=1,N
      IN=I+N
      GO TO (102,101),KDIM
 101  EPS1=EPS(I)
 102  CONTINUE
      IF(EPS1)110,200,110
 110  CONTINUE
      R(I)=0.5*(R(I)+R(IN))
      R1=DABS(R(I)-R(IN))
      R(IN)=R1
      IF(EPS1)120,140,140
 120  R(IN)=RFLAG
      IF(R(I))130,140,130
  130 R(IN)=DABS(R1/R(I))
 140  IF(R(IN)-ABS(EPS1))200,160,160
 160  KFLAG=2
 200  CONTINUE
      FLAG=P
      GO TO (777,201),KFLAG
 201  IF(P-PMAX)800,900,900
 800  CALL EXTERN(K,N,X,F,R,EPS,FLAG)
      IF(FLAG)777,999,999
 999  CONTINUE
 900  FLAG=0.
 777  RETURN
      END

      SUBROUTINE PERIOD(KR,P,H,LP,ALPHA)
      IMPLICIT REAL*8(A-H,O-Z)
      DATA AMAX/0.3D0/, H1/1./
      A=H
      IF(A)1,1,2
   1  A=H1
   2  AK=KR
      ALPHA=A*LP/P**(1./AK)
      IF(ALPHA-AMAX)4,4,3
   3  ALPHA=AMAX
   4  RETURN
      END

C      ===============================================================


      SUBROUTINE DSIMPS(A1,B1,H1,REPS1,AEPS1,FUNCT,X,AI,AIH,AIABS)
C SIMPS
C A1,B1 -THE LIMITS OF INTEGRATION
C H1    -AN INITIAL STEP OF INTEGRATION
C REPS1,AEPS1 - RELATIVE AND ABSOLUTE PRECISION OF INTEGRATION
C FUNCT -A NAME OF FUNCTION SUBPROGRAM FOR CALCULATION OF INTEGRAND +
C X - AN ARGUMENT OF THE INTEGRAND
C AI - THE VALUE OF INTEGRAL
C AIH- THE VALUE OF INTEGRAL WITH THE STEP OF INTEGRATION
C AIABS- THE VALUE OF INTEGRAL FOR MODULE OF THE INTEGRAND
C THIS SUBROGRAM CALCULATES THE DEFINITE INTEGRAL WITH THE RELATIVE OR
C ABSOLUTE PRECISION BY SIMPSON+S METHOD WITH THE AUTOMATICAL CHOICE
C OF THE STEP OF INTEGRATION
C IF AEPS1    IS VERY SMALL(LIKE 1.E-17),THEN CALCULATION OF INTEGRAL
C WITH REPS1,AND IF REPS1 IS VERY SMALL (LIKE 1.E-10),THEN CALCULATION
C OF INTEGRAL WITH AEPS1
C WHEN  AEPS1(REPS1(0. THEN CALCULATION WITH THE CONSTANT STEP H1
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION F(7),P(5)
      H=DSIGN(H1,B1-A1)
      S=DSIGN(1.D0,H)
      A=A1
      B=B1
      AI=0.D0
      AIH=0.D0
      AIABS=0.D0
      P(2)=4.D0
      P(4)=4.D0
      P(3)=2.D0
      P(5)=1.D0
      IF(B-A) 1,2,1
    1 REPS=DABS(REPS1)
      AEPS=DABS(AEPS1)
      DO 3 K=1,7
  3   F(K)=10.D16
      X=A
      C=0.D0
      F(1)=FUNCT(X)/3.D0
    4 X0=X
      IF((X0+4.D0*H-B)*S) 5,5,6
    6 H=(B-X0)/4.D0
      IF(H) 7,2,7
    7 DO 8 K=2,7
  8   F(K)=10.D16
      C=1.D0
    5 DI2=F(1)
      DI3=DABS(F(1))
      DO 9 K=2,5
      X=X+H
      IF((X-B)*S) 23,24,24
   24 X=B
   23 IF(F(K)-10.D16) 10,11,10
   11 F(K)=FUNCT(X)/3.D0
   10 DI2=DI2+P(K)*F(K)
    9 DI3=DI3+P(K)*DABS(F(K))
      DI1=(F(1)+4.D0*F(3)+F(5))*2.D0*H
      DI2=DI2*H
      DI3=DI3*H
      IF(REPS) 12,13,12
   13 IF(AEPS) 12,14,12
   12 EPS=DABS((AIABS+DI3)*REPS)
      IF(EPS-AEPS) 15,16,16
   15 EPS=AEPS
   16 DELTA=DABS(DI2-DI1)
      IF(DELTA-EPS) 20,21,21
   20 IF(DELTA-EPS/8.D0) 17,14,14
   17 H=2.D0*H
      F(1)=F(5)
      F(2)=F(6)
      F(3)=F(7)
      DO 19 K=4,7
  19  F(K)=10.D16
      GO TO 18
   14 F(1)=F(5)
      F(3)=F(6)
      F(5)=F(7)
      F(2)=10.D16
      F(4)=10.D16
      F(6)=10.D16
      F(7)=10.D16
   18 DI1=DI2+(DI2-DI1)/15.D0
      AI=AI+DI1
      AIH=AIH+DI2
      AIABS=AIABS+DI3
      GO TO 22
   21 H=H/2.D0
      F(7)=F(5)
      F(6)=F(4)
      F(5)=F(3)
      F(3)=F(2)
      F(2)=10.D16
      F(4)=10.D16
      X=X0
      C=0.D0
      GO TO 5
   22 IF(C) 2,4,2
    2 RETURN
         END






