      SUBROUTINE FUNINT(K,X,AX,BX,XM,EPS,R)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC    
C                                                                             C
C      This program performs numerical evaluation of functional integrals     C
C      with Wiener measure from the real functional of exponential form       C
C                                                                             C
C                                                                             C
C      The program uses subroutine MIKOR for calculation of the               C
C      multidimensional Riemann integral by the Korobov method                C
C                                                                             C
C                                                                             C
C      The program uses external module DSIMPS for calculation of the         C
C      integral over the auxiliary variable tau by the Simpson's method       C
C                                                                             C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
     
      IMPLICIT REAL*8(A-H,O-Z)
      EXTERNAL FX
      DIMENSION X(K),AX(K),BX(K),F(1),RES(1),DX(2),REPS(1)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      
      
      KK=K
      PI=4.D0*DATAN(1.D0)
      EPS1=DABS(EPS)
      N=K-1

C       Definition of parameters for calculation of
C       (N+1)-dimensional integral using MIKOR 

      KR=K               ! total multiplicity
      M=MM               ! number of integrals
      DO 10 J=1,2        !
      DX(J)=DDX(J)       ! initial number of integration points 
10    CONTINUE           !
      L=LL               ! periodization order
      H=HH               ! boundary layer thickness
      DO 20 I=1,N        !
      AX(I)=-DABS(XM)    ! lower integration limits for U
      BX(I)=DABS(XM)     ! upper integration limits for U
20    CONTINUE           !
      AX(KK)=-1.D0       ! lower integration limits for V
      BX(KK)=1.D0        ! upper integration limits for V
                         
      REPS(1)=-EPS1      ! relative accuracy
      
      CALL MIKOR(KR,M,X,F,RES,DX,L,H,REPS,FLAG,FX,AX,BX)
      
      R=0.5D0*RES(1)/DSQRT(2.D0*PI)**N
      
      RETURN
      END

C      Definition of integrand for MIKOR
      SUBROUTINE FX(K,M,X,F,R,EPS,FLAG)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(K),F(M),R(M)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      
      N=K-1
      
      ARG=0.D0
      DO 30 I=1,N
      ARG=ARG+X(I)**2
30    CONTINUE
      ARG=-0.5D0*ARG

      E=DEXP(ARG)

C      Calculation of the function at the necessary points 
      FF=FUNC(K,X)
      
      F(1)=E*FF
      
      RETURN
      END

C      Definition of the finctional F
      FUNCTION FUNC(K,X)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(K)
      COMMON/CONST/KK,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/U(20),V(1)
      EXTERNAL FUN1,FUN2
      
      A1=0.D0
      B1=1.D0
      H1=1.D-2
      REPS1=1.D-4
      AEPS1=1.D-10
      
      N=K-1
      DO 40 I=1,N
      U(I)=X(I)
40    CONTINUE
      V(1)=X(K)

      CALL DSIMPS(A1,B1,H1,REPS1,AEPS1,FUN1,T1,AI1,AIH1,AIABS1)
      
      A2=0.D0
      B2=1.D0
      H2=1.D-2
      REPS2=1.D-4
      AEPS2=1.D-10
      
      CALL DSIMPS(A2,B2,H2,REPS2,AEPS2,FUN2,T2,AI2,AIH2,AIABS2)
      
      E=DEXP(-BB*AI2)
      
      FUNC=AI1*E
      
      RETURN
      END
      
C      Calculation of the integral part of the functional
      FUNCTION FUN1(T)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/CONST/K,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/UU(20),VV(1)
      
      N=K-1
      V=VV(1)
      
      SGV=DSIGN(1.D0,V)
      RVT=RHO(V,T)
      
      SN=0.D0
      UN=0.D0
      
      DO 50 J=1,N
      DJ=J
      SJPT=DSIN(DJ*PI*T)
      CJPV=DCOS(DJ*PI*V)
      SN=SN+SJPT*CJPV/DJ
      UN=UN+UU(J)*SJPT/DJ
50    CONTINUE

      SN=SN*2.D0*SGV/PI
      UN=UN*DSQRT(2.D0)/PI
      
      RSV=RVT-SN+UN

      FUN1=F1(RSV)

      RETURN
      END

C      Calculation of exponential part of the functional
      FUNCTION FUN2(T)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/CONST/K,PI,EPS1
      COMMON/PAR/BB,DDX(2),LL,MM,HH
      COMMON/AUX/UU(20),VV(1)
      
      N=K-1
      V=VV(1)
      
      SGV=DSIGN(1.D0,V)
      RVT=RHO(V,T)
      
      SN=0.D0
      UN=0.D0
      
      DO 60 J=1,N
      DJ=J
      SJPT=DSIN(DJ*PI*T)
      CJPV=DCOS(DJ*PI*V)
      SN=SN+SJPT*CJPV/DJ
      UN=UN+UU(J)*SJPT/DJ
60    CONTINUE

      SN=SN*2.D0*SGV/PI
      UN=UN*DSQRT(2.D0)/PI
      
      RSV=RVT-SN+UN
      
      FUN2=F2(RSV)

      RETURN
      END

C      Auxilary function for functional integration
      FUNCTION RHO(V,T)
      IMPLICIT REAL*8(A-H,O-Z)
      
      AV=DABS(V)
      SGV=DSIGN(1.D0,V)
        
      IF (T.LE.AV) GOTO 1 
      R=(1.d0-T)*SGV
      GOTO 2
1     CONTINUE
      R=-T*SGV
2     RHO=R
      
      RETURN
      END
      
